function [sqrtLx,sqrtLy,Sxy,Lxy] = InitBivarGauss(Rcirc)
% Preprocessing for simulation of stationary bivariate Gaussian 
% series, W[n]=(X[n],Y[n]), n=0,1,...,N-1, assuming E(W[n]) = (0,0).
% The inputed auto- and cross-covariance sequences are assumed to be
% circularized. 
%
% Usage:
%  [sqrtLx,sqrtLy,Sxy,Lxy] = InitBivarGauss(Rcirc)
% 
% Inputs:
%  Rcirc	circularized covariance structure
% 
% Outputs:
%  sqrtLx   square root of the auto-spectrum (DFT of Rcirc{1,1})
%  sqrtLy   square root of the auto-spectrum (DFT of Rcirc{2,2})
%  Sxy      cross-spectrum
%  Lxy      DFT of Rcirc{1,2}
%
% Copyright (c) Hannes Helgason, ENS Lyon, 2009


% Calculate the eigenvalues of the circularized auto-covariance sequences.
% The eigenvalues should be real-valued, taking real-part in case of
% numerical errors
Lx = fft(Rcirc{1,1});
Ly = fft(Rcirc{2,2});
% check whether properly circularized
if norm(imag(Lx))>0.001,
   warning('Imaginary part of DFT(Rcirc{1,1}) seems to be non-zero. Is the covariance properly circularized?');
end
if norm(imag(Ly))>0.001,
   warning('Imaginary part of DFT(Rcirc{2,2}) seems to be non-zero. Is the covariance properly circularized?');
end
Lx = real(Lx);
Ly = real(Ly);

% Check whether the entries in Lx and Ly are positive, 
% otherwise the auto-covariance is not well-defined
if sum(Lx<=0) > 0,
    error('DFT of the covariance sequence in Rcirc{1,1} has to be positive!');
elseif sum(Ly<=0) > 0,
    error('DFT of the covariance sequence in Rcirc{2,2} has to be positive!');
else
    sqrtLx = sqrt(Lx);
    sqrtLy = sqrt(Ly);
end

% Calculate the cross-spectrum of the processes X and Y
Lxy = fft(Rcirc{1,2});

% the cross-spectrum:
Sxy = zeros(1,length(Lxy));
tmp = sqrtLx.*sqrtLy;
Sxy = Lxy./tmp; 

% Check whether the entries in Sxy are smaller than 2 in absolute value, 
% otherwise the auto-covariance is not well-defined
if max(abs(Sxy)) > 2,
    eps = max(abs(Sxy)) - 2;
    error(sprintf('The cross-spectrum exceeds allowed bound (i.e., 2) by %0.4f.',eps));
end