function [sqrtLambda,crossSpec,Lambda] = InitMultivarGauss(Rcirc)
% Preprocessing for simulation of stationary multivariate Gaussian 
% series, (X1[n],X2[n],...,XP[n]), n=0,1,...,N-1, assuming E(Xk[n]) = 0
% The inputed auto- and cross-covariance sequences are assumed to be
% circularized. 
% Outputs the auto- and cross-spectrums for the resulting circulant sequences.
%
% Usage:
%  [sqrtLambda,crossSpec,Lambda] = InitMultivarGauss(Rcirc)
% 
% Inputs:
%  Rcirc	circularized covariance structure
% 
% Outputs:
%  sqrtLambda  
%       sqrtLambda(k,:) is the square root of the auto-spectrum 
%       for the circulant embedding of R{k,k}
%       It is a matrix of size (P x M), where M is the length of
%       each circulant embedding covariance sequence
%  crossSpec   
%       crossSpec{k,j}, k<j, is the cross-spectrum for the circulant 
%       embeddings of R{k,j}, R{k,k}, and R{j,j}
%  Lambda  the DFT of Rcirc{k,j}
%
% Copyright (c) Hannes Helgason, ENS Lyon, 2009

P = size(Rcirc,1); % number of components
M = length(Rcirc{1,1});

% Calculate the DFT of the circularized covariance sequences.
% Recall that they can be calculated using the DFT (not normalized with
% sqrt(N)).
Lambda = cell(P,P); % initialize cell structure
for k=1:P,
  for j=k:P,
    Lambda{k,j} = fft(Rcirc{k,j});
  end
end

% Check whether the entries in Lambda{k,k} are positive, 
% otherwise the auto-covariance is not well-defined
sqrtLambda = zeros(P,M);
for k=1:P,
	tmp = Lambda{k,k};
	if norm(imag(tmp))>0.001,
        warning(sprintf('Imaginary part of DFT(Rcirc{%d,%d}) seems to be non-zero. Is the covariance properly circularized?',k,k));
    end
    % The auto-spectrum should be real-valued, taking real-part in case of
    % numerical errors
    tmp = real(tmp);
	if sum(tmp<0) > 0,
        error(sprintf('DFT of the covariance sequence in Rcirc{1,1} has to be positive!',k,k));
    end
	sqrtLambda(k,:) = sqrt(tmp);
end

% Calculate the cross-spectrum of the processes Xk and Xj, k<j
crossSpec = cell(P,P);
for k=1:P,
  for j=(k+1):P,
    Lxy = Lambda{k,j};
    Sxy = zeros(1,length(Lxy));
    tmp = sqrtLambda(k,:).*sqrtLambda(j,:);
    nonzeroInd = (tmp~=0);
    Sxy(nonzeroInd) = Lxy(nonzeroInd)./tmp(nonzeroInd); 
    crossSpec{k,j} = Sxy;
  end
end
