function HermirPath
% HermirPath:

% The variable MYHERMIR below should be the path to the location 
% of HERMIR in the file system.

% To hardcode the path to HERMIR, uncomment and edit
% the following line by setting the variable MYHERMIR 
% to the absolute the path where HERMIR is stored on your system:
%MYHERMIR = '/Users/kolmogorov/Code/Hermir'; % NOTE: Do not include trailing slash

% HERMIR can be started from the MATLAB command prompt,
% (one has to be located in the HERMIR folder) by running
% this script (i.e., run 'HermirPath'), with the following
% line uncommented:
MYHERMIR = pwd; % uncomment this line if path is hardcoded


disp('-- STARTING HERMIR --')
disp('HERMIR is a set of MATLAB functions for synthesis of');
disp('Gaussian and non-Gaussian multivariate time series');
disp('Adding HERMIR to MATLAB path...')
disp('Location of HERMIR is assumed to be:');
disp(['  '  MYHERMIR]);


MYHERMIR = [MYHERMIR filesep]; % adding a directory separator

% Add folders to path:

% Folder with code for generating Gaussian processes
dirname = 'GaussGen';
myaddpath(MYHERMIR,dirname);

% Folder with code for generating non-Gaussian processes
dirname = 'NonGaussGen';
myaddpath(MYHERMIR,dirname);

% Folder with mex-routines
dirname = ['mex' filesep 'Hermite'];

% Folder with code with models for generating covariance sequences
dirname = 'CovarGen';
myaddpath(MYHERMIR,dirname);

% Folder with demos
dirname = 'Demo';
myaddpath(MYHERMIR,dirname);

% Folder with Utilities
dirname = 'Utilities';
myaddpath(MYHERMIR,dirname);

% Try to compile mex-routines
CompileMex(MYHERMIR)


function myaddpath(relPath,name)
% Function for adding folder to path.
% Issues a warning message if things go wrong
fname = [relPath name];

if exist(fname),
    disp(['Adding folder: ' name]);
    addpath(fname);
else
	msg = ['Could not add directory ' name ' to path. '...
         'Is the variable MYHERMIR in HermirPath.m set correctly? '...
         'Does the folder ' fname ' exist on your machine?'];
	warning(msg);
end



function CompileMex(packageRootDir)
% CompileMex -- checks whether mex-files have been compiled and compiles them if needed

MEXCOMPILED = 1;

MEXROOT = strcat(packageRootDir,'mex',filesep);

% The list of mex-files to compile is in mexFiles where
% each row in the string array should be
%'NAME-OF-FILE' 'ABSOLUTE-LOCATION-OF-DIR' 'FILE-EXTENSION'
%'DESTINATION-DIR'
%
% FILE-EXTENSION is cpp for C++ code or c for C code
% NAME-OF-FILE is the file name without extension
% ABSOLUTE-LOCATION-OF-DIR is where the code resides
% DESTINATION-DIR is where the directory the executable should be copied
% to. 
% Note: When a MEX-file and an M-file have the same name and are in the 
% same folder, MATLAB will call the MEX-file

mexFiles = {...
'BlockDiagMultiplication' strcat(MEXROOT,'Utils') 'c' strcat(packageRootDir,'Utilities');...
'GHermiteRule' strcat(MEXROOT,'Hermite') 'c' strcat(packageRootDir,'NonGaussGen');...
'HermitePolyval' strcat(MEXROOT,'Hermite') 'c' strcat(packageRootDir,'NonGaussGen');...
%'FactLapack' strcat(MEXROOT,'Utils') 'c' strcat(packageRootDir,'Utilities');...
};

nFiles = size(mexFiles,1);

% Check if MEX files are installed
for k=1:nFiles,
  % check if mex-file exists on MATLAB's search path
  if exist(mexFiles{k,1})~=3,
    MEXCOMPILED = 0;
    break;
  end
end

% If MEX files are not installed, try to install them
if ~MEXCOMPILED,
  disp('MEX files are not installed')
  disp('Compiling MEX files...')
  
  currDir = pwd; % original directory
  for k=1:nFiles,
    cd(mexFiles{k,2});
    fullFileName = sprintf('%s%s%s.%s',mexFiles{k,2},filesep,mexFiles{k,1},mexFiles{k,3});
    disp(sprintf('Compiling: %s',fullFileName));
    try,
      eval(sprintf('mex %s.%s -outdir %s',mexFiles{k,1},mexFiles{k,3},mexFiles{k,4}));
      
    catch
      disp(sprintf('CompileMex: Error occured while compiling MEX file: %s',fullFileName))
      disp('Try running "mex -help" to see if mex compiler is properly installed.')
      % go back to package root directory
      cd(currDir);
    end
    % go back to original directory
    cd(currDir);
  end
end



% Copyright (c) Hannes Helgason, ENS Lyon, 2009
