function [fwdSeq, revSeq] = CovarSequence(coeff1,coeff2,zeropadLength)
% Generate coefficient for the forward relation of covariances
% and use series reversion to get a sequence for the backward relation
%
% Usage
%   [fwdSeq, revSeq] = CovarSequence(coeffs1, coeffs2,zeropadLength)
%   [fwdSeq, revSeq] = CovarSequence(coeffs1, coeffs2)
%   [fwdSeq, revSeq] = CovarSequence(coeffs1)
% Inputs
%   coeffs1, coeff2  	coefficients c_m, m=1,...,M, of the two Hermite expansions
%           			Y_j = \sum_{m=0}^M c_m H_m(X), j=1,2
%   			where c_m=coeffsj(m+1), X~N(0,1)
%   			If only coeffs1 is provided, coeff2=coeff1 will be assumed
%   zeropadLength       length of trailing coeffients in reversion of fwdSeq to be set to 0,
%                       if omitted, the value will be set to 0.
%			In cases where the length of coefficient sequence
%			is short, this will provide greater accuracy for the reversion
%			sequence.
%			See also documentation for reversion.m
%
% Output
%   fwdSeq		coefficients for the forward relation, i.e.,
%				rY= \sum_{m=1}^M fwdSeq(m)*rX^m
%			where fwdSeq(m) = coeffs1(m)*coeffs2(m)*m!
%   revSeq  coefficients from series reversion (note that this can be an approximation)
%				rX \approx \sum_{m=1}^M revSeq(m)*rY^m
%PEND! This function requires coeffs1(1) and coeffs2(1) to be nonzero. Make this more general to
% deal with Hermite expansion with c_m = 0, for all 1<=m<=K, K>=1
%
% Copyright 2009 (c), Hannes Helgason, ENS de Lyon

if nargin < 3,
  zeropadLength = 0;
  if nargin < 2,
    coeff2 = coeff1;
  end
end
% force the sequences to be row vectors
coeff1 = reshape(coeff1,1,length(coeff1));
coeff2 = reshape(coeff2,1,length(coeff2));

% make the length of the sequences equal by adding zeros to the shorter vector
if length(coeff1)>length(coeff2),
  nZerosToAdd = length(coeff1)-length(coeff2);
  coeff2 = [coeff2 zeros(1,nZerosToAdd)];
else
  % length(coeff2)<=length(coeff2)
  nZerosToAdd = length(coeff2)-length(coeff1);
  coeff1 = [coeff1 zeros(1,nZerosToAdd)];
end

% Calculate fwdSeq(m) = coeffs1(m)*coeffs2(m)*m!
mvec = 0:(length(coeff1)-1);

% m! can be quite big so using gammaln and logarithms to do the calc.
lnfwdSeq = log(abs(coeff1))+log(abs(coeff2))+gammaln(mvec+1);
fwdSeq = sign(coeff1).*sign(coeff2).*exp(lnfwdSeq);
% omit c_m for m=0
fwdSeq = fwdSeq(2:end);

% do series reversion
revSeq = reversion(fwdSeq,zeropadLength);
