function [Y,RYapprox,K,funch,approxFlag] = GenMVnGaussUsingExactMap(RY,nameList,paramList,N)
% Generates multivariate stationary non-Gaussian series from a list of
% covariance mappings for which one has analytic formulas
%
% Usage:
%   [Y,RYapprox,K,funch] = GenMVnGaussUsingExactMap(RY,nameList,paramList,N)
% Inputs:   
%   RY      The targeted covariance of non-Gaussian series.
%           The length of the sequences has to be greater than N
%   N       The desired length of the synthesized series
%           Does not need to be equal to length of the covariance
%           sequences
%   nameList  parameter for choosing the transforms (see also
%           documentation for MapAutoCovar.m). This parameter can be
%           either:
%           1) string of transform name, same tranform will be used for all
%              components
%           2) string array of transform names, nameList{p} corresponds to
%              the name of the transform for component p
%   paramList  parameters corresponding to the transforms in nameList. This
%           parameter can be either of the two:
%           1) vector, same parameters will be used for all components
%           2) cell array of vectors, paramList{p} are the parameters used
%              for component p
% Outputs:
%   Y       realization of the non-Gaussian series with marginal defined by
%           the function with handle funch and covariance RYapprox
%   RYapprox resulting covariance of the synthesized non-Gaussian
%           series
%   K       number of independent copies of multivariate stationary
%           Gaussian series used in the mapping
%   funch   cell array of function handles, funch{p} corresponds to mapping
%           for coordinate p
%   approxFlag  true if approximation was needed, otherwise false
%
% Copyright (c) 2010, Hannes Helgason, ENS de Lyon

% Map RY to RX
[RX,K,funch] = MapCovar(RY,nameList,paramList);
% NOTE: RX is not necessarily a valid covariance structure

P = length(RX); % number of components in the series

% Synthesize Gaussian series, use `best covariance approximation' if
% RX from mapping is not non-negative definite
X = cell(1,P);

% Initialize X
for p=1:P,
    X{p} = zeros(K,N);
end

for kk=1:floor(K/2),
    [Xcopy1,Xcopy2,RXapprox,approxFlag] = GenMultivarGaussBestApprox(RX,N);
	% Xcopy1 and Xcopy2 are independent copies of the multivariate Gaussian process 
    for p=1:P,
        X{p}(2*kk-1,:) = Xcopy1(p,:);
        X{p}(2*kk,:) = Xcopy2(p,:);        
    end
end
if rem(K,2)>0,
    % add one more component since K is odd
    [Xcopy1,bogus,RXapprox,approxFlag] = GenMultivarGaussBestApprox(RX,N);
    for p=1:P,
        X{p}(K,:) = Xcopy1(p,:);
    end
end
% Map RXapprox to get the resulting covariance of the synthesized 
% non-Gaussian series
forward = true; % for choosing forward mapping rX->rY
RYapprox = MapCovar(RXapprox,nameList,paramList,forward);

% Transform Gaussian series
Y = zeros(P,N);
for p=1:P,
    Y(p,:) = funch(X{p});
end
