function [Y,rYapprox,K,funch,approxFlag] = GenUnivarNonGaussUsingExactMap(rY,name,params,N)
% Generates univariate stationary non-Gaussian series from a list of
% covariance mappings for which one has analytic formulas
%
% Usage:
%   [Y,rYapprox,K,funch,approxFlag] = GenUnivarNonGaussUsingExactMap(rY,name,params,N)
% Inputs:
%   "name" and "params" are used to choose the transform F, see
%   documentation for MapAutoCovar.m
%   rY      The targeted covariance of non-Gaussian series.
%           NOTE: This sequence will be normalized to match the variance
%           of the marginal set by 'name' and 'params'
%           The length of the sequences has to be greater than N
%   N       The desired length of the synthesized series
%           Can be smaller than the length of the covariance sequence
% Outputs:
%   Y       realization of the non-Gaussian series with marginal defined by
%           the function with handle funch and covariance rYapprox
%   rYapprox resulting covariance of the synthesized non-Gaussian
%           series
%   K       number of independent copies of stationary Gaussian series used in the mapping
%   funch   function handle for the mapping f
%   approxFlag  true if approximation was needed, otherwise false
%


if iscell(rY),
    % the covariance generators, such as CovarMAR1,
    % return cell arrays, even though there is only one component (P=1)
	rY = rY{1};
end

% Map rY to rX
% Find the variance of the marginal for normalization
[varmarg,K,funch] = MapAutoCovar(1,name,params,true);
rYnormalized = varmarg/rY(1) * rY;
    
rX = MapAutoCovar(rYnormalized,name,params);
% NOTE: rX is not necessarily a valid covariance sequence


% Synthesize Gaussian series, use `best covariance approximation' if
% fX from mapping is not non-negative definite
rXcell = cell(1); rXcell{1} = rX; % put in cell sincd it is required below
% Initialize X
X = zeros(K,N);

% initalization for Gaussian series
[rXapproxCell,approxFlag,BmArray] = InitStepMultivarGaussBestApprox(rXcell,N);

for kk=1:floor(K/2),
    [Xcopy1,Xcopy2] = SynthStepMultivarGaussBestApprox(BmArray,N);
	% Xcopy1 and Xcopy2 are independent copies of the multivariate Gaussian
	% process 
	X(2*kk-1,:) = Xcopy1;
	X(2*kk,:) = Xcopy2;        
end
if rem(K,2)>0,
    % add one more component since K is odd
    [Xcopy1,Xcopy2] = SynthStepMultivarGaussBestApprox(BmArray,N);
    X(K,:) = Xcopy1;
end
% Map rXapprox to get the resulting covariance of the synthesized 
% non-Gaussian series
rXapprox = rXapproxCell{1};
forward = true; % for choosing forward mapping rX->rY
rYapprox = MapAutoCovar(rXapprox,name,params,forward);

% Transform Gaussian series
Y = funch(X);
