function [Y,rYapprox,coeffs,X,rXapprox,BmArray,approxFlag] = GenUnivarNonGaussUsingHermite(rY,funch,N,M,xbound,zeropadLength)
% Generates univariate stationary non-Gaussian series using Hermite expansion to calculate
% covariance mappings
%
% Usage:
%   [Y,rYapprox,coeffs,X,rXapprox] = GenUnivarNonGaussUsingHermite(rY,funch,N,M,xbound,zeropadLength)
%   [Y,rYapprox,coeffs,X,rXapprox] = GenUnivarNonGaussUsingHermite(rY,funch,N,M,xbound)
%   [Y,rYapprox,coeffs,X,rXapprox] = GenUnivarNonGaussUsingHermite(rY,funch,N,M)
%   [Y,rYapprox,coeffs,X,rXapprox] = GenUnivarNonGaussUsingHermite(rY,funch,N)
% Inputs:
%   rY      The targeted covariance of non-Gaussian series.
%           NOTE: This sequence will be normalized to match the variance
%           of the marginal set by 'name' and 'params'
%           The length of the sequences has to be greater than N
%   funch   function handle for the non-linear transform f in Y=f(X),
%           Example: funch = @(x) chi2cdf(normcdf(x),1), 
%                    This will make Y chi-squared with one degree of freedom           
%   N       The desired length of the synthesized series
%           Does not need to be equal to length of the covariance
%           sequence
%   M       The order of the Hermite expansion to use for calculating 
%           the covariance relationship, default value is 15
%   xbound  for configuring the interval used in calculating the Hermite coefficients,
%           if omitted, the default value of HermiteCoeffExpansion will be used
%   zeropadLength  use this to increase the number of terms in the reversion series, 
%           if omitted, default value is zero
%PEND: Is zeropadLength really needed?
% Outputs:
%   Y       realization of the non-Gaussian series with marginal defined by
%           the function with handle funch and covariance rYapprox
%   rYapprox resulting covariance of the synthesized non-Gaussian
%           series.            
%   coeffs  numerically calculated Hermite coefficients for funch
%   X       the Gaussian sequence used in the synthesis (i.e., Y=funch(X)) 
%   rXapprox  the covariance sequence used in the synthesis of the Gaussian sequence X
%
%  NOTE: 
%    The targeted covariance will be renormalized by a factor Var(Y)/r^Y[0],
%    where Y=f(X) and r^Y[0] is the inputed variance.
%
%PEND! A lot of the code below is similar to GenUnivarNonGaussUsingExactMap -- change the code to avoid this.
%
% Copyright (c) 2010, Hannes Helgason, ENS de Lyon

if nargin<6,
  zeropadLength = 0;
  if nargin<4,
    M = 15;
  end
end  

% Calculate the Hermite expansion of the function
if nargin<5,
  % xbound is not set
  coeffs = HermiteCoeffExpansion(funch,M);
else
  % xbound is set
  coeffs = HermiteCoeffExpansion(funch,M,xbound);
end

if iscell(rY),
    % the covariance generators, such as CovarMAR1,
    % return cell arrays, even though there is only one component (P=1)
	rY = rY{1};
end


% Calculate the coefficients in the forward covariance relationship,
%   rY= \sum_{m=1}^M fwdSeq(m)*rX^m
% and in the reversion series, which can be used to invert the above relationship,
% i.e.,
%   rX \approx \sum_{m=1}^M revSeq(m)*rY^m
% NOTE! This series does not necessarily give the inverse for all rY, but for
% some neighborhood around rY=0
% NOTE: The current version of the code requires c_1~=0, i.e., coeffs(2)~=0
[fwdSeq, revSeq] = CovarSequence(coeffs, coeffs, zeropadLength);

% Map rY to rX
fwdPolyCoeff = fliplr([0 fwdSeq]);
revPolyCoeff = fliplr([0 revSeq]);

% Find the variance of the marginal for normalization
% we get the variance by plugging rX=1
varY = polyval(fliplr([0 fwdSeq]),1);

rYnormalized = varY/rY(1) * rY;

% Do the reversion but force r^X[0]=1, in case reversion does not work there (which is often the case)
rX = polyval(revPolyCoeff,rYnormalized);
rX(1) = 1; % forcing the variance in the Gaussian sequence to be 1

rYreversion = polyval(fwdPolyCoeff,rX);

%PEND! Make this parameter adjustable...
thresh = 1e-4;
indBAD = find(abs(rYnormalized-rYreversion)>thresh);

%for k = 1:length(rYnormalized);
for k=indBAD
     rX2rY = @(t) polyval(fwdPolyCoeff,t)-rYnormalized(k);
     rX(k) = fzero(rX2rY,1);
end

% Synthesize Gaussian series, use `best covariance approximation' if
% rX from mapping is not non-negative definite
rXcell = cell(1); rXcell{1} = rX; % put in cell since it is required below

% initalization for Gaussian series
[rXapproxCell,approxFlag,BmArray] = InitStepMultivarGaussBestApprox(rXcell,N);

%[X,Xcopy,rXapproxCell] = GenMultivarGaussBestApprox(rXcell,N);
% X and Xcopy are independent copies of the multivariate Gaussian

% Map rXapprox to get the resulting covariance of the synthesized 
% non-Gaussian series
rXapprox = rXapproxCell{1};
rYapprox = polyval(fwdPolyCoeff,rXapprox);

% generate the Gaussian series
[X,Xcopy] = SynthStepMultivarGaussBestApprox(BmArray,N);
% Transform Gaussian series
%PEND: Should we use the Hermite expansion for this since in that case rYapprox is exact?
Y = funch(X);
