function coeffs = HermiteCoeffExpansion(funch,M,xbound)
% Numerical calculation of the coefficients in Hermite expansions
%
% Usage:
%   coeffs = HermiteCoeffExpansion(funch,M,xbound)
%   coeffs = HermiteCoeffExpansion(funch,M)
% Inputs:
%   funch   function handle for the function to be expanded
%   M       the highest order of the Hermite polynomial in the expansion,
%           see below
%   xbound  the integration is be done for -xbound<x<xbound,
%           which is okay if the integrand is close to zero and
%           decreasing rapidly enough outside this region.
%           If omitted, the xbound=8 is used
%           REMARK: 
%           If funch corresponds to a function of the form
%           f(x)=NAMEOFDISTinv(normcdf(x)), where NAMEOFDISTinv is a
%           function for the inverse cdf, then one has to take care of the 
%           range of integration since normcdf(x) can be very close to 1
%           for very low x (e.g., normcdf(8)~1-6.7e-16). In this case,
%           MATLAB will could return f(x)=Inf (e.g., Matlab gives
%           chi2inv(normcdf(8.3),1)=Inf)
% Output:
%   coeffs  vector of length M+1 with the coefficients in the expansion, 
%           see below
%
% Comments:
%   The inner-product is defined by
%     <f,g> = \int_{-\infty}^\infty f(x)g(x) w(x)dx
%   where w(x) = exp(-x^2/2)/sqrt(2*pi)
% 
%   The Hermite polynomials can be found by the recursion
%     H_{m+1}(x) = x*H_m(x) - m*H_{m-1}(x), H_{-1}(x)=0, H_0(x)=1
%   They are orthogonal with <H_m,H_m>=m!
%
%   If F is the function corresponding to the function handle funch,
%   then coeff(m+1)=a_m, in the expansion
%     \tilde{F}(x) = \sum_{m=0}^M a_m H_m(x)
%   where
%     a_m = <F,H_m>/m!
%
%PEND! Choosing xbound=8 seems to work well in practice. If it is
% taken larger Matlab starts returning Inf. It remains to investigate
% in which cases can cause significant error; that is, when
% H_m(x)f(x)exp(-x^2/2) does not tend to zero fast enough, in which
% case one might need to handle the regions (xbound,Inf) and (-Inf,-xbound),
% differently. One possibility could be to use an asymptotic approximation
% of f(x).
%
% Copyright (c) Hannes Helgason, ENS Lyon, 2009

if nargin < 3,
  xbound = 8;
end

coeffs = zeros(1,M+1);

% Get the Gaussian-Hermite quadrature rule
NN = 100;
%PEND! NN is hardcoded... should we allow for the possibility of tuning this?
[w,x] = GHermiteRule(NN);
% change to probabilist Hermite
%PEND! Do this within GHermiteRule
w = w/sqrt(pi);
x = sqrt(2)*x;

%PEND! What about using a simple trapezoidal rule instead of Hermite rule?

ind = (x < xbound)&(x>-xbound); 
f = funch(x);
for m=0:M,
    % evaluate integrand at the points in x
    nx = length(x);
    g = zeros(1,nx);
    hx = HermitePolyval(x,m); 
    % calculating  g = f*hx/factorial(m) directly is bad, denominator will cause overflow for big m
    % using logarithm to perform the calculation:
    lnabsg = log(abs(f))+log(abs(hx))-gammaln(m+1);
    g = sign(f).*sign(hx).*exp(lnabsg);
    % store coefficient
    coeffs(m+1) = sum(w(ind).*g(ind));
end
